@@ -91,7 +91,7 @@ sub waitpid {
 }
 
 sub waitall {
-  my $timeout = shift || 864000;       # XXX - 10 days arbitrary limit is not the Perl way.
+  my $timeout = shift || 9E9;  # 285 years should be long enough to wait
   $timeout = 1E-6 if $timeout < 0;
   my $waited_for = 0;
   my $expire = Time() + $timeout ;
@@ -188,7 +188,7 @@ sub _reap {
 # wait on any process
 sub _waitpid_any {
   my ($no_hang,$reap_bg_ok,$timeout) = @_;
-  my $expire = Time() + ($timeout || 864000);
+  my $expire = Time() + ($timeout || 9E9);
   my ($pid, $nactive2, $nalive, $nactive) = _reap($reap_bg_ok);
   unless ($no_hang) {
     while (!isValidPid($pid) && $nalive > 0) {
@@ -236,7 +236,7 @@ sub _active_one_suspended_job {
 # wait on a specific process
 sub _waitpid_target {
   my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-  my $expire = Time() + ($timeout || 864000);
+  my $expire = Time() + ($timeout || 9E9);
   my $job = $ALL_JOBS{$target};
   if (not defined $job) {
     return -1;
@@ -263,7 +263,7 @@ sub _waitpid_target {
 
 sub _waitpid_name {
   my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-  my $expire = Time() + ($timeout || 864000);
+  my $expire = Time() + ($timeout || 9E9);
   my @jobs = Forks::Super::Job::getByName($target);
   if (@jobs == 0) {
     return -1;
@@ -301,7 +301,7 @@ sub _waitpid_name {
 # wait on any process from a specific process group
 sub _waitpid_pgrp {
   my ($no_hang, $reap_bg_ok, $target, $timeout) = @_;
-  my $expire = Time() + ($timeout || 864000);
+  my $expire = Time() + ($timeout || 9E9);
   my ($pid, $nactive) = _reap($reap_bg_ok,$target);
   unless ($no_hang) {
     while (!isValidPid($pid) && $nactive > 0) {
